@@ -14,6 +14,7 @@ import re
 from fnmatch import fnmatch
 from pathlib import Path
 from typing import Union
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -285,7 +286,7 @@ def load_file_content(fil):
 
     Returns
     -------
-    any
+    Any
         Array/json/pandas dataframe depending on format
     """
     if not fil:
@@ -315,6 +316,13 @@ def load_file_content(fil):
     if fil.suffix in ('.yml', '.yaml'):
         with open(fil, 'r') as _fil:
             return yaml.safe_load(_fil)
+    if fil.suffix == '.sparse_npz':
+        try:
+            import sparse
+            return sparse.load_npz(fil)
+        except ModuleNotFoundError:
+            warnings.warn(f'{Path(fil).name} requires the pydata sparse package to load.')
+            return Path(fil)
     return Path(fil)
 
 
@@ -330,7 +338,7 @@ def _ls(alfpath, object=None, **kwargs) -> (list, tuple):
         An ALF object name to filter by
     wildcards : bool
         If true uses unix shell style pattern matching, otherwise uses regular expressions
-    kwargs : dict
+    **kwargs
         Other ALF parts to filter, including namespace, attribute, etc.
 
     Returns
@@ -421,7 +429,7 @@ def exists(alfpath, object, attributes=None, **kwargs) -> bool:
         Wanted attributes
     wildcards : bool
         If true uses unix shell style pattern matching, otherwise uses regular expressions
-    kwargs : dict
+    **kwargs
         Other ALF parts to filter by
 
     Returns
@@ -461,23 +469,23 @@ def load_object(alfpath, object=None, short_keys=False, **kwargs):
     Parameters
     ----------
     alfpath : str, pathlib.Path, list
-        Any ALF path pertaining to the object OR directory containing ALFs OR list of paths
+        Any ALF path pertaining to the object OR directory containing ALFs OR list of paths.
     object : str, list, None
         The ALF object(s) to filter by.  If a directory is provided and object is None, all valid
-        ALF files returned
+        ALF files returned.
     short_keys : bool
         By default, the output dictionary keys will be compounds of attributes, timescale and
         any eventual parts separated by a dot. Use True to shorten the keys to the attribute
-        and timescale
+        and timescale.
     wildcards : bool
-        If true uses unix shell style pattern matching, otherwise uses regular expressions
-    kwargs : dict
-        Other ALF parts to filter by
+        If true uses unix shell style pattern matching, otherwise uses regular expressions.
+    **kwargs
+        Other ALF parts to filter by.
 
     Returns
     -------
     AlfBunch
-        An ALFBunch (dict-like) of all attributes pertaining to the object
+        An ALFBunch (dict-like) of all attributes pertaining to the object.
 
     Examples
     --------
@@ -634,15 +642,27 @@ def save_metadata(file_alf, dico) -> None:
 
 def remove_uuid_file(file_path, dry=False) -> Path:
     """
-    Renames a file without the UUID and returns the new pathlib.Path object
+    (DEPRECATED) Renames a file without the UUID and returns the new pathlib.Path object.
+
+    Parameters
+    ----------
+    file_path : str, pathlib.Path
+        An ALF path containing a UUID in the file name.
+    dry : bool
+        If False, the file is not renamed on disk.
+
+    Returns
+    -------
+    pathlib.Path
+        The new file path without the UUID in the file name.
     """
-    if isinstance(file_path, str):
-        file_path = Path(file_path)
-    name_parts = file_path.name.split('.')
-    if not spec.is_uuid_string(name_parts[-2]):
-        return file_path
-    name_parts.pop(-2)
-    new_path = file_path.parent.joinpath('.'.join(name_parts))
+    warnings.warn(
+        'remove_uuid_file deprecated, use one.alf.files.remove_uuid_string instead',
+        DeprecationWarning)
+    file_path = Path(file_path)
+    new_path = files.remove_uuid_string(file_path)
+    if new_path == file_path:
+        return new_path
     if not dry and file_path.exists():
         file_path.replace(new_path)
     return new_path
@@ -650,8 +670,18 @@ def remove_uuid_file(file_path, dry=False) -> Path:
 
 def remove_uuid_recursive(folder, dry=False) -> None:
     """
-    Within a folder, recursive renaming of all files to remove UUID
+    (DEPRECATED) Within a folder, recursive renaming of all files to remove UUID.
+
+    Parameters
+    ----------
+    folder : str, pathlib.Path
+        A folder to recursively iterate, removing UUIDs from the file names.
+    dry : bool
+        If False renames the files on disk.
     """
+    warnings.warn(
+        'remove_uuid_recursive is deprecated and will be removed in the next release',
+        DeprecationWarning)
     for fn in Path(folder).rglob('*.*'):
         print(remove_uuid_file(fn, dry=dry))
 
