@@ -16,14 +16,11 @@ from pathlib import Path, PurePosixPath
 from urllib.parse import urlsplit
 from typing import Optional, Union, Mapping, List, Iterable as Iter
 
-import numpy as np
 import pandas as pd
-from iblutil.io import parquet
 from iblutil.util import Bunch
 
-import one.alf.io as alfio
 from one.alf.spec import is_session_path, is_uuid_string
-from one.alf.files import get_session_path, add_uuid_string, session_path_parts
+from one.alf.files import get_session_path, add_uuid_string, session_path_parts, remove_uuid_string
 from .util import Listable, ensure_list
 
 
@@ -38,12 +35,12 @@ def recurse(func):
     Parameters
     ----------
     func : function
-        A method to decorate
+        A method to decorate.
 
     Returns
     -------
     function
-        The decorated method
+        The decorated method.
     """
     @functools.wraps(func)
     def wrapper_decorator(*args, **kwargs):
@@ -175,8 +172,6 @@ class ConversionMixin:
             return
 
         # load path from cache
-        if self._index_type() is int:
-            eid = parquet.str2np(eid).tolist()
         try:
             ses = self._cache['sessions'].loc[eid].squeeze()
             assert isinstance(ses, pd.Series), 'Duplicate eids in sessions table'
@@ -218,8 +213,6 @@ class ConversionMixin:
         assert len(sessions) == 1
 
         eid, = sessions.index.values
-        if isinstance(eid, tuple):
-            eid = parquet.np2str(np.array(eid))
         return eid
 
     @recurse
@@ -258,7 +251,7 @@ class ConversionMixin:
         if isinstance(path, str) and path.startswith('http'):
             # Remove the UUID from path
             path = urlsplit(path).path.strip('/')
-            path = alfio.remove_uuid_file(PurePosixPath(path), dry=True)
+            path = remove_uuid_string(PurePosixPath(path))
             session_path = get_session_path(path).as_posix()
         else:
             # No way of knowing root session path parts without cache tables
@@ -313,17 +306,17 @@ class ConversionMixin:
         # FIXME Should be OneAlyx converter only
         if isinstance(record, pd.DataFrame):
             return [self.record2url(r) for _, r in record.iterrows()]
-        if isinstance(record, pd.Series):
+        elif isinstance(record, pd.Series):
             is_session_record = 'rel_path' not in record
             if is_session_record:
                 # NB: This assumes the root path is in the webclient URL
                 session_spec = '{lab}/Subjects/{subject}/{date}/{number:03d}'
                 url = record.get('session_path') or session_spec.format(**record)
                 return webclient.rel_path2url(url)
-            if all(isinstance(x, (int, np.int64)) for x in record.name):
-                uuid, = parquet.np2str(np.array([record.name[-2:]]))
-            else:
-                uuid = ensure_list(record.name)[-1]  # may be (eid, did) or simply did
+            uuid = ensure_list(record.name)[-1]  # may be (eid, did) or simply did
+        else:
+            raise TypeError(
+                f'record must be pandas.DataFrame or pandas.Series, got {type(record)} instead')
 
         session_path, rel_path = record[['session_path', 'rel_path']].to_numpy().flatten()
         url = PurePosixPath(session_path, rel_path)
@@ -353,23 +346,23 @@ class ConversionMixin:
     def eid2ref(self, eid: Union[str, Iter], as_dict=True, parse=True) \
             -> Union[str, Mapping, List]:
         """
-        Get human-readable session ref from path
+        Get human-readable session ref from path.
 
         Parameters
         ----------
         eid : str, uuid.UUID
-            The experiment uuid to find reference for
+            The experiment uuid to find reference for.
         as_dict : bool
-            If false a string is returned in the form 'subject_sequence_yyyy-mm-dd'
+            If false a string is returned in the form 'subject_sequence_yyyy-mm-dd'.
         parse : bool
             If true, the reference date and sequence are parsed from strings to their respective
-            data types
+            data types.
 
         Returns
         -------
         dict, str, list
             One or more objects with keys ('subject', 'date', 'sequence'), or strings with the
-            form yyyy-mm-dd_n_subject
+            form yyyy-mm-dd_n_subject.
 
         Examples
         --------
@@ -400,18 +393,18 @@ class ConversionMixin:
     @recurse
     def ref2eid(self, ref: Union[Mapping, str, Iter]) -> Union[str, List]:
         """
-        Returns experiment uuid, given one or more experiment references
+        Returns experiment uuid, given one or more experiment references.
 
         Parameters
         ----------
         ref : str, dict, list
             One or more objects with keys ('subject', 'date', 'sequence'), or strings with
-            the form yyyy-mm-dd_n_subject
+            the form yyyy-mm-dd_n_subject.
 
         Returns
         -------
         str, list
-            One or more experiment uuid strings
+            One or more experiment uuid strings.
 
         Examples
         --------
@@ -436,18 +429,18 @@ class ConversionMixin:
     @recurse
     def ref2path(self, ref):
         """
-        Convert one or more experiment references to session path(s)
+        Convert one or more experiment references to session path(s).
 
         Parameters
         ----------
         ref : str, dict, list
             One or more objects with keys ('subject', 'date', 'sequence'), or strings with
-            the form yyyy-mm-dd_n_subject
+            the form yyyy-mm-dd_n_subject.
 
         Returns
         -------
         pathlib.Path
-            Path object(s) for the experiment session(s)
+            Path object(s) for the experiment session(s).
 
         Examples
         --------
@@ -475,14 +468,14 @@ class ConversionMixin:
         Parameters
         ----------
         path_str : str
-            A path to a given session
+            A path to a given session.
         as_dict : bool
-            If True a Bunch is returned, otherwise a string
+            If True a Bunch is returned, otherwise a string.
 
         Returns
         -------
         dict, str, list
-            One or more objects with keys ('subject', 'date', 'sequence')
+            One or more objects with keys ('subject', 'date', 'sequence').
 
         Examples
         --------
@@ -507,18 +500,18 @@ class ConversionMixin:
     @staticmethod
     def is_exp_ref(ref: Union[str, Mapping, Iter]) -> Union[bool, List[bool]]:
         """
-        Returns True is ref is a valid experiment reference
+        Returns True is ref is a valid experiment reference.
 
         Parameters
         ----------
         ref : str, dict, list
             One or more objects with keys ('subject', 'date', 'sequence'), or strings with
-            the form yyyy-mm-dd_n_subject
+            the form yyyy-mm-dd_n_subject.
 
         Returns
         -------
-        bool
-            True if ref is valid
+        bool, list of bool
+            True if ref is valid.
 
         Examples
         --------
@@ -538,23 +531,23 @@ class ConversionMixin:
             ref = '{date}_{sequence}_{subject}'.format(**ref)
         elif not isinstance(ref, str):
             return False
-        return re.compile(r'\d{4}(-\d{2}){2}_(\d{1}|\d{3})_\w+').match(ref) is not None
+        return re.compile(r'\d{4}(-\d{2}){2}_(\d{1,3})_\w+').match(ref) is not None
 
     @staticmethod
     @parse_values
     def ref2dict(ref: Union[str, Mapping, Iter]) -> Union[Bunch, List]:
         """
-        Returns a Bunch (dict-like) from a reference string (or list thereof)
+        Returns a Bunch (dict-like) from a reference string (or list thereof).
 
         Parameters
         ----------
         ref : str, list
-            One or more experiment reference strings
+            One or more experiment reference strings.
 
         Returns
         -------
         iblutil.util.Bunch
-            A Bunch in with keys ('subject', 'sequence', 'date')
+            A Bunch in with keys ('subject', 'sequence', 'date').
 
         Examples
         --------
@@ -581,17 +574,26 @@ class ConversionMixin:
         Parameters
         ----------
         ref_dict : dict, Bunch, list, tuple
-            A map with the keys ('subject', 'date', 'sequence')
+            A map with the keys ('subject', 'date', 'sequence').
 
         Returns
         -------
         str, list:
-            An experiment reference string, or list thereof
+            An experiment reference string, or list thereof.
         """
         if isinstance(ref_dict, (list, tuple)):
             return [ConversionMixin.dict2ref(x) for x in ref_dict]
         if not ref_dict:
             return
+        if 'sequence' not in ref_dict and 'number' in ref_dict:
+            ref_dict = ref_dict.copy()
+            ref_dict['sequence'] = ref_dict.pop('number')
+        if 'date' not in ref_dict and 'start_time' in ref_dict:
+            ref_dict = ref_dict.copy()
+            if isinstance(ref_dict['start_time'], str):
+                ref_dict['date'] = ref_dict['start_time'][:10]
+            else:
+                ref_dict['date'] = ref_dict['start_time'].date()
         parsed = any(not isinstance(k, str) for k in ref_dict.values())
         format_str = ('{date:%Y-%m-%d}_{sequence:d}_{subject:s}'
                       if parsed
@@ -607,14 +609,14 @@ def one_path_from_dataset(dset, one_cache):
     Parameters
     ----------
     dset : dict, list
-        Dataset dictionary or list of dictionaries from Alyx rest endpoint
+        Dataset dictionary or list of dictionaries from Alyx rest endpoint.
     one_cache : str, pathlib.Path, pathlib.PurePath
-        The local ONE data cache directory
+        The local ONE data cache directory.
 
     Returns
     -------
     pathlib.Path
-        The local path for a given dataset
+        The local path for a given dataset.
     """
     return path_from_dataset(dset, root_path=one_cache, uuid=False)
 
@@ -627,19 +629,19 @@ def path_from_dataset(dset, root_path=PurePosixPath('/'), repository=None, uuid=
     Parameters
     ----------
     dset : dict, list
-        Dataset dictionary or list of dictionaries from Alyx rest endpoint
+        Dataset dictionary or list of dictionaries from Alyx rest endpoint.
     root_path : str, pathlib.Path, pathlib.PurePath
-        The prefix path such as the ONE download directory or remote http server root
+        The prefix path such as the ONE download directory or remote http server root.
     repository : str, None
         Which data repository to use from the file_records list, defaults to first online
-        repository
+        repository.
     uuid : bool
-        If True, the file path will contain the dataset UUID
+        If True, the file path will contain the dataset UUID.
 
     Returns
     -------
     pathlib.Path, list
-        File path or list of paths
+        File path or list of paths.
     """
     if isinstance(dset, list):
         return [path_from_dataset(d) for d in dset]
@@ -660,22 +662,20 @@ def path_from_filerecord(fr, root_path=PurePosixPath('/'), uuid=None):
     Parameters
     ----------
     fr : dict
-        An Alyx file record dict
+        An Alyx file record dict.
     root_path : str, pathlib.Path
-        An optional root path
+        An optional root path.
     uuid : str, uuid.UUID
-        An optional dataset UUID to add to the file name
+        An optional dataset UUID to add to the file name.
 
     Returns
     -------
     pathlib.Path
-        A filepath as a pathlib object
+        A filepath as a pathlib object.
     """
     if isinstance(fr, list):
         return [path_from_filerecord(f) for f in fr]
-    repo_path = fr['data_repository_path']
-    repo_path = repo_path[repo_path.startswith('/'):]  # remove starting / if any
-    # repo_path = (p := fr['data_repository_path'])[p[0] == '/':]  # py3.8 Remove slash at start
+    repo_path = (p := fr['data_repository_path'])[p[0] == '/':]  # Remove slash at start, if any
     file_path = PurePosixPath(repo_path, fr['relative_path'])
     if root_path:
         # NB: By checking for string we won't cast any PurePaths
