@@ -82,6 +82,31 @@ def get_s3_virtual_host(uri, region) -> str:
     return 'https://' + '/'.join((hostname, *key))
 
 
+def url2uri(data_path, return_location=False):
+    """
+    Convert a generic Amazon virtual host URL to an S3 URI.
+
+    Parameters
+    ----------
+    data_path : str
+        An Amazon virtual host URL to convert.
+    return_location : bool
+        If true, additionally returns the location string.
+
+    Returns
+    -------
+    str
+        An S3 URI with scheme 's3://'.
+    str
+        If return_location is true, returns the bucket location, e.g. 'eu-east-1'.
+    """
+    parsed = urllib.parse.urlparse(data_path)
+    assert parsed.netloc and parsed.scheme and parsed.path
+    bucket_name, _, loc, *_ = parsed.netloc.split('.')
+    uri = f's3://{bucket_name}{parsed.path}'
+    return (uri, loc) if return_location else uri
+
+
 def is_folder(obj_summery) -> bool:
     """
     Given an S3 ObjectSummery instance, returns true if the associated object is a directory.
@@ -164,9 +189,9 @@ def get_s3_from_alyx(alyx, repo_name=REPO_DEFAULT):
     Notes
     -----
     - If no credentials are present in the database, boto3 will use environment config or default
-     AWS profile settings instead.
+      AWS profile settings instead.
     - If there are no credentials for the bucket and the bucket has 'public' in the name, the
-     returned resource will use an unsigned signature.
+      returned resource will use an unsigned signature.
     """
     session_keys, bucket_name = get_aws_access_keys(alyx, repo_name)
     no_creds = not any(filter(None, (v for k, v in session_keys.items() if 'key' in k.lower())))
